﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace DSASearch {
    class Program {
        static void Main(String[] args) {
            ConsoleKey key = mainMenu().Key;
            do {
                switch (key) {
                    case ConsoleKey.D0: return;
                    case ConsoleKey.D1: findSentinel(false); break;
                    case ConsoleKey.D2: findSentinel(true); break;
                    case ConsoleKey.D3: automateSentinel(); break;
                    case ConsoleKey.D4: findHashTable(); break;
                    case ConsoleKey.D5: automateHashTable(); break;
                }
            } while (mainMenu().Key != ConsoleKey.D0);
        }
        static void findSentinel(Boolean ordered) {
            Int32 size = requestArraySize();
            SentinelSearch array = new SentinelSearch(size);
            Int32[] rnd = genRandomNumbers(size);
            foreach (Int32 random in rnd) {
                array.Add(random);
            }
            if (ordered) { array.Sort(); }
            array.Print();
            Console.WriteLine();
            Console.Write("Enter number to find: ");
            String response = Console.ReadLine();
            Int32 numberToFind = Convert.ToInt32(response);
            Console.WriteLine("Number of comparisons made: " + array.FindValue(numberToFind));
        }
        static void findHashTable() {
            Int32 size = requestArraySize();
            CustomHashTable array = new CustomHashTable(size, .3);
            Int32[] rnd = genRandomNumbers(size);
            foreach (Int32 random in rnd) {
                array.Add(random);
            }
            array.Print();
            Console.WriteLine();
            Console.Write("Enter number to find: ");
            String response = Console.ReadLine();
            Int32 numberToFind = Convert.ToInt32(response);
            Console.WriteLine("Number of comparisons made: " + array.FindKey(numberToFind));
        }
        static void automateSentinel() {
            var sb = new StringBuilder();
            for (Int32 i = 100; i < 1001; i += 100) {
                sb.Append(i + "");
                for (Int32 j = 1; j <= 10; j++) {
                    SentinelSearch array = new SentinelSearch(i);
                    Int32[] rndArray = genRandomNumbers(i);
                    foreach (Int32 random in rndArray) {
                        array.Add(random);
                    }
                   var rnd = new Random(Guid.NewGuid().GetHashCode());
                    Int32 numberToFind = rndArray[rnd.Next(0, i - 1)];
                    sb.Append("," + array.FindValue(numberToFind));
                }
                sb.Append(Environment.NewLine);
            }
            File.WriteAllText("sentinel.csv", sb.ToString());
        }
        static void automateHashTable() {
            Double loadFactor = 0;
            do {
                var sb = new StringBuilder();
                for (Int32 i = 100; i < 1001; i += 100) {
                    sb.Append(i + "");
                    for (Int32 j = 1; j <= 10; j++) {
                        CustomHashTable array = new CustomHashTable(i,loadFactor);
                        Int32[] rndArray = genRandomNumbers(i);
                        foreach (Int32 random in rndArray) {
                            array.Add(random);
                        }
                        Int32 numberToFind;
                        if (j == 1) {
                            numberToFind = -1;
                        } else {
                            var rnd = new Random(Guid.NewGuid().GetHashCode());
                            numberToFind = rndArray[rnd.Next(0, i - 1)];
                        }
                        sb.Append("," + array.FindKey(numberToFind));
                    }
                    sb.Append(Environment.NewLine);
                }
                File.WriteAllText("htable-" + loadFactor * 10 + ".csv", sb.ToString());
                loadFactor += .1;
            } while (loadFactor < .4);
        }
        static ConsoleKeyInfo mainMenu() {
            Console.WriteLine("[1] Find Sentinel unordered");
            Console.WriteLine("[2] Find Sentinel ordered");
            Console.WriteLine("[3] Automate Sentinel");
            Console.WriteLine("[4] Find HashTable");
            Console.WriteLine("[5] Automate HashTable");
            Console.WriteLine("[0] Exit");
            Console.Write("Select option: ");
            ConsoleKeyInfo retValue = Console.ReadKey(false);
            Console.WriteLine();
            return retValue;
        }
        static Int32 requestArraySize() {
            Console.Write("Enter array size: ");
            String response = Console.ReadLine();
            Console.WriteLine();
            return Convert.ToInt32(response);
        }
        static Int32[] genRandomNumbers(Int32 size) {
            HashSet<Int32> randoms = new HashSet<Int32>();
            Random rnd = new Random(DateTime.Now.Millisecond);
            do {
                randoms.Add(rnd.Next(1, 9999));
            } while (randoms.Count < size);
            return randoms.ToArray();
        }
	    static void genStatFile(List<List<Int32>> output, Int32 size) {
            using (StreamWriter wr = new StreamWriter("htable-stats.csv")) {
                wr.Write("Insertions");
                for (Int32 i = 0; i < output.Count; i++) {
                    wr.Write(",Trial {0}",i);
                }
                wr.Write(Environment.NewLine);
                for (Int32 i = 0; i < size; i++) {
                    wr.Write(i + 1);
                    for (Int32 j = 0; j < output.Count; j++) {
                        wr.Write(",{0}", output[j][i]);
                    }
                    wr.Write(Environment.NewLine);
                }
            }
        }
    }
}
