﻿using System;

namespace DSASearch {
    class SentinelSearch {
        readonly Int32[] _array;
        Int32 indexToInsert;
        Boolean isSorted;

        public SentinelSearch(Int32 size) {
            // initialize the array size to be bigger than requested one
            _array = new Int32[size + 1];
        }

        public Int32 Length {
            get { return _array.Length - 1; }
        }
        public void Add(Int32 value) {
            if (indexToInsert == _array.Length - 1) { throw new OverflowException("The array is full"); }
            _array[indexToInsert] = value;
            indexToInsert++;
            isSorted = false;
        }
        public Int32 FindValue(Int32 value) {
            // put value to find to the last element of the array
            _array[_array.Length - 1] = value;
            // select appropriate search method depending on sort status
            return isSorted ? findSorted(value) : findUnsorted(value);
        }
        public void Print() {
            for (Int32 i = 0; i < _array.Length - 1; i++) {
                Console.Write(_array[i] + " ");
            }
        }
        public void Sort() {
            // leave the open implementation of sorting as it is not important here.
            Array.Sort(_array, 0, _array.Length - 1);
            isSorted = true;
        }
        Int32 findSorted(Int32 value) {
            Int32 counter = 0;
            // if the array is sorted (ordered) we can check only values that are less or
            // equals to the value in search.
            while (_array[counter] < value) {
                counter++;
            }
            _array[_array.Length - 1] = 0;
            return counter;
        }
        Int32 findUnsorted(Int32 value) {
            Int32 counter = 0;
            // loop over array to find the desired value. If original array does not contain
            // the key, it will be found in the last element of the array, thus we eliminate
            // the requirement to check array boundaries during search. Only value
            // comparison is performed, thus granting O(n) complexity.
            while (_array[counter] != value) {
                counter++;
            }
            _array[_array.Length - 1] = 0;
            return counter;
        }
    }
}
